import path from "path";
import { EventBus } from "./event-bus";
import { GitService } from "./GitService";
import * as vscode from "vscode";

export class Commander {
    constructor(private _gitService: GitService, private eventBus: EventBus) {
        console.log("Commander");
    }

    public async compare(commit: string, file: string) {
        const workspaceFolder = vscode.workspace.workspaceFolders?.[0]?.uri.fsPath;
        if (!workspaceFolder) {
            vscode.window.showErrorMessage('Please open a workspace folder first');
            return;
        }
        try {
            const rawPrevCommit = await this._gitService.getPreviousCommit(commit);
            const previousCommitId = rawPrevCommit?.trim(); // 去除可能的换行符
            if (!previousCommitId) {
                vscode.window.showErrorMessage('Cannot find previous commit');
                return;
            }

            // 编码处理
            const encodedCommit = encodeURIComponent(commit.trim());
            const encodedPrevCommit = encodeURIComponent(previousCommitId);
            const sanitizedWorkPath = workspaceFolder.replace(/\\/g, '/')
                .replace(/'/g, "'")
                .replace(/ /g, " ");
            const sanitizedFilePath = file.replace(/\\/g, '/')
                .replace(/'/g, "'")
                .replace(/ /g, " ");
            // 构造 URI
            const currentUri = this.makeUri(sanitizedFilePath, encodedCommit);

            const previousUri = this.makeUri(sanitizedFilePath, encodedPrevCommit);

            await vscode.commands.executeCommand(
                "vscode.diff",
                previousUri,
                currentUri,
                `Changes: ${path.basename(file)} (${previousCommitId.slice(0, 7)} → ${commit.slice(0, 7)})`
            );
        } catch (error) {
            vscode.window.showErrorMessage(`Failed to open diff: ${error instanceof Error ? error.message : error}`);
        }
    }
    private makeUri(filePath: string, commit: string) {
        const repoPath = this._gitService.repoPath;
        const repoUri = vscode.Uri.file(repoPath);
        const fUri = vscode.Uri.joinPath(repoUri, filePath);
        const params = {
            path: fUri.fsPath,
            ref: commit
        }
        return fUri.with({ scheme: 'git', path: fUri.path, query: JSON.stringify(params) });
    }

    public changeToList() {
        // set state to list
        vscode.commands.executeCommand('setContext', 'gitcommitfilter:viewState', 'list');
        this.eventBus.emit('changeView', 'list');
    }

    public changeToTree() {
        // set state to tree
        vscode.commands.executeCommand('setContext', 'gitcommitfilter:viewState', 'tree');
        this.eventBus.emit('changeView', 'tree');
    }
}